@@ -18,6 +18,10 @@ response_vars.mdl_df <- function(x){
   x%@%"response"
 }
 #' @export
+response_vars.mdl_ts <- function(x){
+  vapply(x$response, rlang::as_string, FUN.VALUE = character(1L))
+}
+#' @export
 response_vars.dcmp_ts <- function(x){
   x%@%"response"
 }
